/*
  Copyright 1996,1997,1999 The University of Rhode Island and The
  Massachusetts Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE. */

// This is the main CGI program which extracts the requested variable from a
// matlab data file. It generates the DDS in-memory from both the matlab data
// file and external file and uses it to locate the variable to process.
//
// ReZa 08/29/96

// $Log: mat_dods.cc,v $
// Revision 1.14  1999/05/04 03:30:52  jimg
// Merged no gnu changes
//
// Revision 1.13  1999/01/21 01:47:17  jimg
// Added call to set filename in the DDS. This is required for proper operation
// of the projection functions.
//
// Revision 1.12.8.1  1999/04/09 05:29:01  brent
// convert String.h code to new standard lib <string>
//
// Revision 1.12  1998/03/20 00:32:22  jimg
// Changed calls to the set_mime_*() to match the new parameters
//
// Revision 1.11  1997/12/30 23:56:43  jimg
// Fixed errant comments.
//
// Revision 1.10  1997/12/30 22:49:32  jimg
// Changed from old style filter to an implementation that uses the DODSFilter
// class.
//
// Revision 1.9  1997/10/04 00:33:22  jimg
// Release 2.14c fixes
//
// Revision 1.8.2.1  1997/09/03 22:22:04  jimg
// Changed to work with the modified MatLabUtil.cc file; global objects are
// now pointers.
//
// Revision 1.8  1997/08/05 22:44:01  jimg
// Changed type of option_char in `option_char = getopt(...)' from char to
// int. This fixes a bug that shows up on the SGI (where char is unsigned).
//
// Revision 1.7  1997/06/05 17:15:43  jimg
// Rewrote main() so that it uses functions added to cgi_util.cc for the 2.14
// release. These function help to unify the *_dods filters so that they all
// respond more uniformly.
//
// Revision 1.6  1997/05/01 20:56:07  jimg
// Replaced out ouput code with 2.13-style compressing code
//
// Revision 1.5  1997/05/01 18:36:07  jimg
// Added configureation header.
// Merged changes from interim 2.1.2 version onto main trunk.
//
// Revision 1.4  1997/02/07 17:56:13  reza
// Fixed so null constraint will return the entire data set now
//
// Revision 1.3  1996/12/13 00:14:43  jimg
// Fixed so that when the filter is called with only one argument (a null
// constraint expression) it will work.
//
// Revision 1.2  1996/11/13 05:13:09  reza
// Added complex matrix capability.
//
// Revision 1.1  1996/10/31 14:43:42  reza
// First release of DODS-matlab servers.

#include "config_mat.h"

static char *rcsid not_used = "$Id: mat_dods.cc,v 1.14 1999/05/04 03:30:52 jimg Exp $";

#include <iostream>
#include <string>

#include "cgi_util.h"
#include "DDS.h"
#include "DODSFilter.h"

extern bool read_descriptors(DDS &dds, string filename, string *err_msg);

int 
main(int argc, char * argv[])
{
    DDS dds;
    DODSFilter df(argc, argv);
	
    if (!df.OK()) {
	df.print_usage();
	return 1;
    }

    if (df.version()) {
	df.send_version_info();
	return 0;
    }

    dds.filename(df.get_dataset_name());

    // Read the dataset descriptor in memory
    string errMsg;
    if(!read_descriptors(dds, df.get_dataset_name(), &errMsg)){
	ErrMsgT("mat_dods: could not read the DDS information");
	//client error message
	Error e(unknown_error, errMsg);
	set_mime_text(cout, dods_error);
	e.print(cout);
	return 1;
    }

    if (!df.read_ancillary_dds(dds))
	return 1;

    if (!df.send_data(dds, stdout))
	return 1;

    return 0;
}
